/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic
 * Semiconductor ASA.Terms and conditions of usage are described in detail
 * in NORDIC SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRENTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision: 2211 $
 */

/** @file
 * @brief Enhanced ShockBurst Primary Receiver example
 * @defgroup esb_prx_example Enhanced ShockBurst Primary Receiver (PRX) example
 * @{
 * @ingroup nrf_examples
 *
 * @brief This example monitors for data and writes the first byte (byte 0) of the
 * received payloads to P0.
 *
 * The example shows the minimum required setup for receiving packets from a
 * primary transmitter (PTX) device.
 *
 * The following default radio parameters are being used:
 * - RF channel 2
 * - 2 Mbps data rate
 * - RX address 0xE7E7E7E7E7 (pipe 0) and 0xC2C2C2C2C2 (pipe 1)
 * - 1 byte CRC
 *
 * The project @ref esb_ptx_example can be used as a counterpart for transmitting the data.
 *
*/

//lint -e717
//lint -e534
//lint -e714
//lint -e783

#include "nrf24lu1p.h"
#include "stdint.h"
#include "hal_nrf.h"
#include "usb.h"
#include "usb_cmds.h"
#include "config.h"
#include "bootloader.h"
//#include "hal_clk.h"

// Global variables
uint8_t payload[3];
uint8_t debug;
bool trasmetti;
extern bool packet_received;
static bool volatile radio_busy;


void main()
{
  usb_init();
	
	// Enable radio SPI
  RFCTL = 0x10;

  // Set P0 as output
  P0DIR = 0;

  // Enable the radio clock
  RFCKEN = 1;

  // Enable RF interrupt
  RF = 1;
  // Enable global interrupt
  EA = 1;

	// Imposta il canale 70
	hal_nrf_set_rf_channel(70);
	
  
	// Trasmettitore
	// Configure radio as primary receiver (PTX)
  //hal_nrf_set_operation_mode(HAL_NRF_PRX);

  // Set payload width to 3 bytes
  //hal_nrf_set_rx_payload_width((int)HAL_NRF_PIPE0, 3);

  // Power up radio
  hal_nrf_set_power_mode(HAL_NRF_PWR_UP);

  // Enable receiver
  //CE_HIGH();
  
  packet_received = false;
	trasmetti = false;
	//debug2 = 0;
	//forceSize = false;
	P0 = 0;

  for(;;){
		if (USBF){
			USBF = 0;
			usb_irq();
			if(packet_received){
				parse_commands();
				packet_received = false;
      }
			//debug ^= 2;
    }
		if (trasmetti){
			hal_nrf_write_tx_payload(payload, 3U);		// Write payload to radio TX FIFO
			CE_PULSE();																// Toggle radio CE signal to start transmission
			radio_busy = true;
			// Wait for radio operation to finish
			//debug ^= 4;
			while (radio_busy){}
		}
		
		//debug ^= 1;	// toggle bit 0
		//P0 = debug;
  }
}

// Radio interrupt
NRF_ISR()
{
  uint8_t irq_flags;

  // Read and clear IRQ flags from radio
  irq_flags = hal_nrf_get_clear_irq_flags();
	/*
  // If data received
  if((irq_flags & (1<<(uint8_t)HAL_NRF_RX_DR)) > 0)
  {
    // Read payload
    while(!hal_nrf_rx_fifo_empty())
    {
      hal_nrf_read_rx_payload(payload);
    }

    // Write received payload[0] to port 0
    // P0 = payload[0];
  }
	*/
	switch(irq_flags)
  {
    // Transmission success
    case (1 << (uint8_t)HAL_NRF_TX_DS):
      radio_busy = false;
      // Data has been sent
      break;
    // Transmission failed (maximum re-transmits)
    case (1 << (uint8_t)HAL_NRF_MAX_RT):
      // When a MAX_RT interrupt occurs the TX payload will not be removed from the TX FIFO.
      // If the packet is to be discarded this must be done manually by flushing the TX FIFO.
      // Alternatively, CE_PULSE() can be called re-starting transmission of the payload.
      // (Will only be possible after the radio irq flags are cleared)
      hal_nrf_flush_tx();
      radio_busy = false;
      break;
    default:
      break;
  }
}


/** @} */
